#!/usr/bin/env python
import json
import os
import subprocess
import sys
import time
import traceback

from distutils.dir_util import copy_tree
from glob import glob

from mozdevice import ADBAndroid

def main():
    adb = '/builds/worker/android-sdk-linux/platform-tools/adb'
    taskcluster_debug = '*'

    task_cwd = os.getcwd()
    print('Current working directory: {}'.format(task_cwd))

    print('Task working directory')
    try:
        print(subprocess.check_output(['ls', '-laR', '.'],
                                      stderr=subprocess.STDOUT))
    except subprocess.CalledProcessError:
        print(traceback.format_exc())

    artifacts = os.path.join(task_cwd, 'artifacts')
    if not os.path.exists(artifacts):
        raise Exception('{} does not exist.'.format(artifacts))

    try:
        print(subprocess.check_output([adb, 'devices', '-l'],
                                      stderr=subprocess.STDOUT))
    except subprocess.CalledProcessError:
        print(traceback.format_exc())

    try:
        for f in glob('/tmp/adb.*.log'):
            print('\n{}:\n'.format(f))
            with open(f) as afile:
                print(afile.read())
    except:
        print(traceback.format_exc())

    with open('/builds/taskcluster/scriptvars.json') as scriptvars:
        scriptvarsenv = json.loads(scriptvars.read())

    payload = json.loads(sys.stdin.read())

    print('payload = {}'.format(json.dumps(payload, indent=4)))

    env = dict(os.environ)
    env.update(payload['env'])

    print('Bitbar test run: https://mozilla.testdroid.com/#testing/device-session/{}/{}/{}'.format(
        scriptvarsenv['TESTDROID_PROJECT_ID'],
        scriptvarsenv['TESTDROID_BUILD_ID'],
        scriptvarsenv['TESTDROID_RUN_ID']))

    if 'PATH' in os.environ:
        path = os.environ['PATH']
    else:
        path = '/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games:/usr/local/games:/snap/bin'

    path += ':/builds/worker/android-sdk-linux/tools/bin:/builds/worker/android-sdk-linux/platform-tools'

    env['PATH'] = os.environ['PATH'] = path
    env['NEED_XVFB'] = 'false'
    env['DEVICE_NAME'] = scriptvarsenv['TC_DEVICE_NAME']
    env['DEVICE_SERIAL'] = scriptvarsenv['TC_DEVICE_SERIAL']
    env['HOST_IP'] = scriptvarsenv['TC_CONTAINER_IP']

    if 'HOME' not in env:
        env['HOME'] = '/builds/worker'
        print('setting HOME to {}'.format(env['HOME']))

    if 'TASKCLUSTER_WORKER_TYPE' not in env:
        print('TASKCLUSTER_WORKER_TYPE is missing.')
        return 1

    if 'WORKSPACE' not in env:
        env['WORKSPACE'] = os.path.join(env['HOME'], 'workspace')
        print('script.py: WORKSPACE={}'.format(env['WORKSPACE']))
    workspace = env['WORKSPACE']
    if not os.path.exists(workspace):
        print('script.py: mkdir {}'.format(workspace))
        os.mkdir(workspace)

    print('Connecting to Android device {}'.format(env['DEVICE_SERIAL']))
    try:
        device = ADBAndroid(device=env['DEVICE_SERIAL'])

        # clean up the device.
        device.rm('/data/local/tests', recursive=True, force=True, root=True)
        device.rm('/data/local/tmp/*', recursive=True, force=True, root=True)
        device.rm('/data/local/tmp/xpcb', recursive=True, force=True, root=True)
        device.rm('/sdcard/tests', recursive=True, force=True, root=True)

        max_attempts = 3
        wait_time = 60
        device_ip_address = None
        for attempt in range(max_attempts):
            print('Attempt {} to get device ip address'.format(attempt))
            device_ip_address = device.get_ip_address()
            if device_ip_address:
                break
            time.sleep(wait_time)

        if device_ip_address:
            print('Device ip address {}'.format(device_ip_address))
        else:
            sys.stderr.write('Unable to obtain ip address for device %s' % env['DEVICE_SERIAL'])
            return 1

        env['DEVICE_IP'] = device_ip_address

    except:
        sys.stderr.write(traceback.format_exc())
        return 1

    if taskcluster_debug:
        env['DEBUG'] = taskcluster_debug

    print('environment = {}'.format(json.dumps(env, indent=4)))

    oauthpath = '/builds/taskcluster/oauth.txt'
    with open(oauthpath, 'w') as oauthfile:
        oauthfile.write("taskcluster_clientId = {}\n".format(
            scriptvarsenv['TASKCLUSTER_CLIENT_ID']))
        oauthfile.write("taskcluster_accessToken = {}\n".format(
            scriptvarsenv['TASKCLUSTER_ACCESS_TOKEN']))

    os.chdir(workspace)
    script = os.path.basename(payload['context'])
    subprocess.check_output(['curl', '-O', payload['context']],
                            stderr=subprocess.STDOUT)
    subprocess.check_output(['chmod', '+x', script],
                            stderr=subprocess.STDOUT)

    # Use a login shell to get the required environment for the unit
    # test scripts to detect sys.executable correctly.  Execute the
    # context script in the directory /builds/worker/workspace.
    args = ['bash', '-l', '-c', ' '.join(payload['command'])]
    print(' '.join(args))
    rc = None
    proc = subprocess.Popen(args,
                            env=env,
                            cwd=workspace,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT)
    while rc == None:
        line = proc.stdout.readline()
        sys.stdout.write(line)
        rc = proc.poll()

    os.chdir(task_cwd)

    # Copy the requested directories to the appropriate
    # location in the task's artifacts directory.
    for artifact in payload['artifacts']:
        artifact_location = os.path.join('artifacts', artifact['name'])
        os.makedirs(artifact_location)
        copy_tree(artifact['path'], artifact_location)
        # Temporarily remove empty files due to
        # https://github.com/taskcluster/taskcluster-worker/issues/341
        for f in glob(os.path.join(artifact_location, '*')):
            if os.path.isfile(f) and os.path.getsize(f) == 0:
                os.unlink(f)

    try:
        subprocess.check_call([adb, 'kill-server'],
                              stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError:
        print(traceback.format_exc())

    print('netstat -aop\n%s\n\n' % subprocess.check_output(['netstat', '-aop'],
                                                           stderr=subprocess.STDOUT))


    print('payload.py exitcode {}'.format(rc))
    if rc == 0:
        return 0
    return 1

if __name__ == "__main__":
    sys.exit(main())
